import { world, system } from "@minecraft/server";

/** @type {import("@minecraft/server").BlockCustomComponent} */
const camoDoorComp = {
    onPlayerInteract({ block, dimension }) {
        const isOpen = block.permutation.getState("gao:open");
        const newOpen = !isOpen;
        const newPerm = block.permutation.withState("gao:open", newOpen);
        const sound = newOpen ? "random.door_open" : "random.door_close";

        block.setPermutation(newPerm);

        const above = block.above();
        const below = block.below();

        for (const other of [above, below]) {
            if (other?.typeId === "gao:camo_door") {
                other.setPermutation(other.permutation.withState("gao:open", newOpen));
            }
        }       

        dimension.playSound(sound, block.center(), {
            pitch: 0.9,
            volume: 0.8,
        });
    }
};

world.beforeEvents.worldInitialize.subscribe(({ blockComponentRegistry }) => {
    blockComponentRegistry.registerCustomComponent(
        "gao:camo_door",
        camoDoorComp
    );
});

const blocks = [
    "minecraft:stone", "minecraft:dirt", 
    "gao:ghost_block", "gao:camo_door",
    "minecraft:oak_planks", "minecraft:cobblestone",
    "minecraft:mossy_cobblestone", "minecraft:stone_bricks",
    "minecraft:mossy_stone_bricks", "minecraft:gravel",
    "minecraft:andesite", "minecraft:diorite",
    "minecraft:granite", "minecraft:brick_block",
    "minecraft:bookshelf", "minecraft:deepslate",
    "minecraft:netherrack"
];

const blockMap = {
    "minecraft:stone": "stone",
    "minecraft:dirt": "dirt",
    "minecraft:oak_planks": "oak_planks",
    "minecraft:cobblestone": "cobblestone",
    "minecraft:mossy_cobblestone": "mossy_cobblestone",
    "minecraft:stone_bricks": "stone_bricks",
    "minecraft:mossy_stone_bricks": "mossy_stone_bricks",
    "minecraft:gravel": "gravel",
    "minecraft:andesite": "andesite",
    "minecraft:diorite": "diorite",
    "minecraft:granite": "granite",
    "minecraft:brick_block": "bricks",
    "minecraft:bookshelf": "bookshelf",
    "minecraft:deepslate": "deepslate",
    "minecraft:netherrack": "netherrack"
};

world.afterEvents.playerPlaceBlock.subscribe(event => {
    const { block } = event;

    if (block.typeId === "gao:ghost_block" || block.typeId === "gao:camo_door") {
        const directions = block.typeId === "gao:ghost_block"
                ? [
                    block.below(),
                    block.above(),
                    block.north(),
                    block.east(),
                    block.south(),
                    block.west()
                ]
                : [
                    block.below(),
                    block.above().above(), 
                    block.north(),
                    block.east(),
                    block.south(),
                    block.west()
                ];

        for (const adjacent of directions) {
            if (!blocks.includes(adjacent.typeId)) continue;

            if (adjacent.typeId === block.typeId) {
                const state = adjacent.permutation.getState("gao:block");
                const perm = block.permutation.withState("gao:block", state);
                block.setPermutation(perm);
                break;
            } else if (blockMap[adjacent.typeId]) {
                const gaoBlock = blockMap[adjacent.typeId];
                const perm = block.permutation.withState("gao:block", gaoBlock);
                block.setPermutation(perm);
                break;
            }
        }
    }

    if (block.typeId === "gao:camo_door") {
        const above = block.above();
        if (above && above.typeId === "minecraft:air") {
            above.setType("gao:camo_door");
            above.setPermutation(block.permutation);
        } else {
            system.run( () => {
                block.setType("minecraft:air");
            });
        }
    }
});

world.beforeEvents.playerBreakBlock.subscribe(({ block }) => {
    const above = block.above();
    const below = block.below();

    if (block.typeId === "gao:camo_door") {
        system.run( () => {
            let target;
            if (above.typeId === "gao:camo_door") target = above;
            else if (below.typeId === "gao:camo_door") target = below;

            if (target) {
                target.setType("minecraft:air");
            }
        });
    }
    
});